// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.messageBus;

import com.microsoft.azure.servicebus.ExceptionPhase;
import com.microsoft.azure.servicebus.IMessage;
import com.microsoft.azure.servicebus.IMessageHandler;
import com.microsoft.azure.servicebus.SubscriptionClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.CompletableFuture;

public class MessageHandler implements IMessageHandler {

    private final static Logger LOGGER = LoggerFactory.getLogger(MessageHandler.class);
    private final SubscriptionClient receiveClient;
    private ProcessNotification processNotification;

    public MessageHandler(SubscriptionClient client, ProcessNotification processNotification) {
        this.receiveClient = client;
        this.processNotification = processNotification;
    }

    @Override
    public CompletableFuture<Void> onMessageAsync(IMessage message) {
        try {
            this.processNotification.performNotification(message, receiveClient.getSubscriptionName());
            return this.receiveClient.completeAsync(message.getLockToken());
        } catch (Exception e) {
            LOGGER.error("Unable to process the Notification : " + e);
            return this.receiveClient.abandonAsync(message.getLockToken());
        }
    }

    @Override
    public void notifyException(Throwable throwable, ExceptionPhase exceptionPhase) {
        LOGGER.error("{} - {}", exceptionPhase, throwable.getMessage());
    }
}
