package org.opengroup.osdu.notification.di;

import org.junit.Assert;
import org.junit.Test;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.notification.ISubscriptionService;
import org.opengroup.osdu.core.common.notification.SubscriptionAPIConfig;
import org.opengroup.osdu.notification.provider.azure.di.SubscriptionFactoryAzure;

public class SubscriptionFactoryAzureTest {

    @Test
    public void constructor_should_work_when_config_is_not_null() {
        SubscriptionAPIConfig subscriptionAPIConfig = SubscriptionAPIConfig.builder().build();
        subscriptionAPIConfig.setApiKey("apiKey");
        subscriptionAPIConfig.setRootUrl("rootUrl");
        SubscriptionFactoryAzure subscriptionFactoryAzure = new SubscriptionFactoryAzure(subscriptionAPIConfig, null);
        Assert.assertNotNull(subscriptionFactoryAzure);
    }

    @Test(expected = NullPointerException.class)
    public void constructor_should_throw_npe_if_config_is_null() {
        SubscriptionFactoryAzure subscriptionFactoryAzure = new SubscriptionFactoryAzure(null, null);
    }

    @Test
    public void create_subscriptionService_instance_dpsHeaders_is_not_null() {
        DpsHeaders dpsHeaders = new DpsHeaders();
        SubscriptionAPIConfig subscriptionAPIConfig = SubscriptionAPIConfig.builder().build();
        subscriptionAPIConfig.setApiKey("apiKey");
        subscriptionAPIConfig.setRootUrl("rootUrl");
        SubscriptionFactoryAzure subscriptionFactoryAzure = new SubscriptionFactoryAzure(subscriptionAPIConfig, null);
        ISubscriptionService subscriptionService = subscriptionFactoryAzure.create(dpsHeaders);
        Assert.assertNotNull(subscriptionService);
    }

    @Test(expected = NullPointerException.class)
    public void create_subscriptionService_should_throw_npe_if_headers_is_null() {
        SubscriptionAPIConfig subscriptionAPIConfig = SubscriptionAPIConfig.builder().build();
        subscriptionAPIConfig.setApiKey("apiKey");
        subscriptionAPIConfig.setRootUrl("rootUrl");
        SubscriptionFactoryAzure subscriptionFactoryAzure = new SubscriptionFactoryAzure(subscriptionAPIConfig, null);
        ISubscriptionService subscriptionService = subscriptionFactoryAzure.create(null);
    }
}