// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.messageBus;

import com.microsoft.azure.servicebus.Message;
import com.microsoft.azure.servicebus.SubscriptionClient;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.opengroup.osdu.notification.provider.azure.messageBus.MessageHandler;
import org.opengroup.osdu.notification.provider.azure.messageBus.ProcessNotification;

import java.util.UUID;

import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class MessageHandlerTest {

    private static final UUID uuid = UUID.randomUUID();
    private static final String subscriptionName = "TestSubscription";

    @InjectMocks
    private MessageHandler messageHandler;

    @Mock
    private ProcessNotification processNotification;

    @Mock
    private SubscriptionClient subscriptionClient;

    @Mock
    private Message message;

    @BeforeEach
    public void init() {
        when(message.getLockToken()).thenReturn(uuid);
        when(subscriptionClient.getSubscriptionName()).thenReturn(subscriptionName);
    }

    @Test
    public void shouldInvokeCompleteAsync() throws Exception {
        lenient().doNothing().when(processNotification).performNotification(message, subscriptionName);
        messageHandler.onMessageAsync(message);
        verify(subscriptionClient, times(1)).completeAsync(uuid);
        verify(processNotification, times(1)).performNotification(message, subscriptionClient.getSubscriptionName());
    }

    @Test
    public void shouldInvokeAbandonAsyncWhenProcessNotificationThrowsException() throws Exception {
        doThrow(new Exception()).when(processNotification).performNotification(message, subscriptionName);
        messageHandler.onMessageAsync(message);
        verify(subscriptionClient, times(1)).abandonAsync(uuid);
        verify(processNotification, times(1)).performNotification(message, subscriptionClient.getSubscriptionName());
    }
}
