// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.messageBus;

import com.microsoft.azure.servicebus.SubscriptionClient;
import com.microsoft.azure.servicebus.primitives.ServiceBusException;
import org.junit.Assert;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.opengroup.osdu.azure.servicebus.ISubscriptionClientFactory;
import org.opengroup.osdu.notification.provider.azure.messageBus.SubscriptionClientFactImpl;

import static org.junit.Assert.fail;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
public class SubscriptionClientFactoryTest {
    @InjectMocks
    private SubscriptionClientFactImpl subsClientFactory;

    @Mock
    private SubscriptionClient subscriptionClient;

    @Mock
    private ISubscriptionClientFactory subscriptionClientFactory;

    private static final String sbTopic = "testTopic";
    private static final String sbSubscription = "testSubscription";
    private static final String dataPartition = "testPartition";
    private final String EXCEPTION_NOT_THROWN = "Should Throw Exception";
    private final String SERVICE_BUS_EXCEPTION = "Unable to retrieve client info from Service Bus";

    @Test
    public void subscriptionClientShouldNotBeNull() throws ServiceBusException, InterruptedException {
        when(subscriptionClientFactory.getClient(dataPartition, sbTopic, sbSubscription))
                .thenReturn(subscriptionClient);
        SubscriptionClient result = subsClientFactory.getSubscriptionClient(dataPartition, sbTopic, sbSubscription);
        assertNotNull(result);
        assertEquals(subscriptionClient, result);
    }

    @Test
    public void shouldThrowExceptionWhenSubscriptionClientThrowsException() throws ServiceBusException, InterruptedException {
        when(subscriptionClientFactory.getClient(dataPartition, sbTopic, sbSubscription))
                .thenThrow(new ServiceBusException(false, SERVICE_BUS_EXCEPTION));
        try {
            subsClientFactory.getSubscriptionClient(dataPartition, sbTopic, sbSubscription);
            fail(EXCEPTION_NOT_THROWN);
        } catch (Exception e) {
            Assert.assertNotNull(e);
            Assert.assertEquals( SERVICE_BUS_EXCEPTION, e.getMessage());
        }
    }
}
