// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.opengroup.osdu.notification.provider.azure.util.AppProperties;
import org.opengroup.osdu.notification.provider.azure.util.AzureServiceAccountValidatorImpl;

import java.io.UnsupportedEncodingException;

import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.initMocks;

public class AzureServiceAccountValidatorImplTest {
    private static String eventGridId = "eventgridid";

    @Mock
    private AppProperties appProperties;

    @InjectMocks
    @Spy
    private AzureServiceAccountValidatorImpl sut;

    @Before
    public void setup() {
        initMocks(this);
        when(this.appProperties.getEventGridId()).thenReturn(eventGridId);
    }

    @Test
    public void should_returnFalse_isValidServiceAccount() throws UnsupportedEncodingException {
        // Set Up
        boolean expected = false;
        Algorithm algorithm = Algorithm.HMAC256("secret");
        String token = JWT.create()
                .withIssuer("auth0")
                .sign(algorithm);
        // Act
        boolean observed = this.sut.isValidPublisherServiceAccount(token);

        // Assert
        Assert.assertEquals(expected, observed);
    }
}
