#!/bin/bash

# Exit as soon as a command fails
set -e

BUILD_DEFINITIONNAME="{{definitionName}}"
BUILD_SOURCEBRANCHNAME="{{sourceBranchName}}"
BUILD_BUILDNUMBER="{{buildNumber}}"
SCRIPT_DIR=$(dirname $0)

echo "This script will deploy the app to $GCLOUD_PROJECT"

if [ "$GCLOUD_PROJECT" = "" ]
then
    export GCLOUD_PROJECT=$(gcloud config get-value project)
fi

echo "  Environment = $ENVIRONMENT"
echo "  Project = $GCLOUD_PROJECT"
echo "  Build definition = $BUILD_DEFINITIONNAME"
echo "  Source branch = $BUILD_SOURCEBRANCHNAME"
echo "  Build number = $BUILD_BUILDNUMBER"
echo "  Directory = $SCRIPT_DIR"

echo "Print Directory contents"
ls $SCRIPT_DIR

VERSION_NUMBER=$BUILD_SOURCEBRANCHNAME-$BUILD_BUILDNUMBER
VERSION_NUMBER=$(echo "$VERSION_NUMBER" | tr _ - | tr '[:upper:]' '[:lower:]')
## UGLY.. removes '.' from string.
VERSION_NUMBER=${VERSION_NUMBER//.}
SERVICE_NAME="os-notification"

count=$(gcloud app services list --project $GCLOUD_PROJECT | grep $SERVICE_NAME | wc -l)
if [ $count -gt 0 ]; then
  CURRENT_VERSION=$(gcloud app services describe $SERVICE_NAME --project $GCLOUD_PROJECT --format=json | jq --raw-output '.split.allocations | keys[0]')
else
  CURRENT_VERSION=""
fi
echo "Current version = $CURRENT_VERSION"
echo "Version to be deployed = $VERSION_NUMBER"

if [ "$VERSION_NUMBER" != "$CURRENT_VERSION" -o "$BUILD_FORCE_DEPLOY" = "true" -o "$BUILD_FORCE_DEPLOY" = "True" -o "$BUILD_FORCE_DEPLOY" = "TRUE" ]
then
    if [ "$BUILD_SOURCEBRANCHNAME" = "master" -o "$BUILD_FORCE_DEPLOY" = "true" -o "$BUILD_FORCE_DEPLOY" = "True" -o "$BUILD_FORCE_DEPLOY" = "TRUE" ]
    then
        APPYAML=app.$ENVIRONMENT.yaml
        cat $APPYAML

        # If the version $VERSION_NUMBER already exists, but not serving traffic, gcloud app deploy will fail. In this case, simply start that version and migrate traffic to it
        doesVersionExist=`gcloud app versions list --service=$SERVICE_NAME --project=$GCLOUD_PROJECT | grep $VERSION_NUMBER | wc -l`
        if [ "$doesVersionExist" -eq "1" ]
        then
            echo "Starting version $VERSION_NUMBER"
            gcloud app versions start $VERSION_NUMBER --service=$SERVICE_NAME --project=$GCLOUD_PROJECT --quiet
            echo "Migrating all traffic to version $VERSION_NUMBER"
            gcloud app services set-traffic $SERVICE_NAME --splits $VERSION_NUMBER=1 --project=$GCLOUD_PROJECT --quiet
        else 
            echo "Deploying service to Google project $GCLOUD_PROJECT"
            gcloud app deploy --quiet --version=$VERSION_NUMBER --project=$GCLOUD_PROJECT $APPYAML
            echo "Service deployed successfully"
            ## delete the current version which serves no more traffic if the deployment succeeded, no way to stop an auto-scaled version in app engine standard
            if [ $count -gt 0 ]; then
                echo "Deleting previous version: $CURRENT_VERSION."
                gcloud app versions delete $CURRENT_VERSION --project=$GCLOUD_PROJECT --quiet
            fi
        fi
    else
        echo "Not deploying the application because build source branch was not master"
    fi
else
    echo "Not deploying the application because $VERSION_NUMBER is already deployed and force deploy flag is not set to true"
fi