/*
 * Copyright 2021 Google LLC
 * Copyright 2021 EPAM Systems, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.opengroup.osdu.notification.provider.reference.util;

import com.google.api.client.googleapis.auth.oauth2.GoogleIdToken;
import com.google.api.client.googleapis.auth.oauth2.GoogleIdTokenVerifier;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.jackson2.JacksonFactory;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.notification.provider.interfaces.IServiceAccountValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class GoogleServiceAccountValidatorImpl implements IServiceAccountValidator {

  private final NetHttpTransport netHttpTransport = new NetHttpTransport();
  private final JacksonFactory jacksonFactory = new JacksonFactory();

  @Autowired
  private JaxRsDpsLog log;
  @Autowired
  private AppProperties appConfig;
  @Autowired
  private GoogleServiceAccountValidatorGenerator verifierGenerator;

  @Override
  public boolean isValidPublisherServiceAccount(String jwt) {
    return isValidServiceAccount(jwt, this.appConfig.getPubSubServiceAccountEmail());
  }

  @Override
  public boolean isValidServiceAccount(String jwt, String userIdentity, String... googleAudiences) {
    GoogleIdTokenVerifier verifier = this.verifierGenerator.getVerifier(this.netHttpTransport,
        this.jacksonFactory, googleAudiences);
    try {
      GoogleIdToken idToken = verifier.verify(jwt);
      if (idToken != null) {
        GoogleIdToken.Payload payload = idToken.getPayload();

        String email = payload.getEmail();
        Boolean emailVerified = payload.getEmailVerified();

        return (emailVerified && (email.equalsIgnoreCase(userIdentity)));
      } else {
        return false;
      }
    } catch (Exception e) {
      this.log.error("Error when validating google id token", e);
      return false;
    }
  }
}
